/*******************************************************************************
 * RGB_Led_Controller.c - Main Code File
 * Autore: Matteo Garia
 * Data Creazione: 19/04/2012
 * Versione: 1.1 (07/10/2012)
 * ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "RGB_Led_Controller.h"

void interrupt isr(){
    if (TMR0IF)
    {
//Ricarico TMR0 con il preload stabilito
        TMR0=TMR0_PRELOAD;
//Imposto il flag di avvenuto interrupt
        main_int = 1;
//Azzero l'interrupt flag di TMR0
        TMR0IF=0;
    }

    if(RCIF)
    {
//Leggo il buffer seriale
        rx_chars[rx_counter] = RCREG;
//Incremento il contatore caratteri
        rx_counter++;
//NOTA: il flag di interrupt si azzera in automatico dopo la lettura
    }

}

void main(){
    SetupSFR();
    SetupADC();
    init_comms();
//Impostazioni di default all'alimentazione del dispositivo
    power = OFF;
    pcMode = PWR_OFF;

    while(TRUE)
    {
        if (main_int)
        {
//Verifico se la comunicazione seriale si  interrotta (timeout)
            if(rx_counter > 0){com_timeout++;}
            if(com_timeout == TIMEOUT){
                rx_counter = 0;
                com_timeout = 0;
                ERROR;
            }
//Passo a decodificare i comandi ricevuti (se presenti)
            if(rx_counter == 4)
            {
                com_timeout = 0;
                PcCommands();
            }
//Decremento il contatore programma
            programCounter--;
//Chiamo la funzione di acquisizione analogica
            if (programCounter == 0)
            {
                AdConversion();
                RESET_PROGRAM_COUNTER;

            }
//Leggo lo stato dei pulsanti, quando la conversione  completa
            if(btnIsValid){
                AnalogBTN(btnValue);
                btnIsValid = 0;
            }
//Leggo la posizione del potenziometro
            if(programCounter == 66 && power == ON){
//Se la modalit  AUTO, il potenziometro modifica la durata dell'effetto
//Viceversa se MANUAL, il potenziometro modifica il colore
                if (mode == AUTO){
                    if(autoTimer > (potValue)){
                        effectValue ++;
                        autoTimer = 0;
                        if (effectValue == 1023){effectValue = 0;}
                    }
                }
                else
                {
                    effectValue = 1023 - potValue;
                }
            }
//Richiamo la funzione dell'effetto scelto, per calcolare i colori
            if(programCounter == 132 && power == ON){
                if (effect == RAINBOW){
                    RainbowRGB(effectValue);
                }
                if (effect == SOFT){
                    SoftRGB(effectValue);
                }
            }

//Imposto i pwm a seconda della modalit operativa scelta
            switch (pcMode){
                case AUX_BAR_PC_MODE_ON:
                    SoftwarePwm(pcLeftRed,pcLeftGreen,pcLeftBlue,
                                pcRightRed,pcRightGreen,pcRightBlue,
                                pcAuxRed,pcAuxGreen,pcAuxBlue);
                    break;
                case PWR_OFF:
                    SoftwarePwm(0,0,0,0,0,0,0,0,0);
                    break;
                case PC_MODE_ON:
                    SoftwarePwm(pcLeftRed,pcLeftGreen,pcLeftBlue,
                                pcRightRed,pcRightGreen,pcRightBlue,
                                effectRed,effectGreen,effectBlue);
                    break;
                case PC_MODE_OFF:
                    SoftwarePwm(effectRed,effectGreen,effectBlue,
                                effectRed,effectGreen,effectBlue,
                                effectRed,effectGreen,effectBlue);
                    break;
            }

            if(power == ON){autoTimer ++;}
            main_int=0;

        }
    }

}

void SoftwarePwm(unsigned char pwm0, unsigned char pwm1, unsigned char pwm2,
                 unsigned char pwm3, unsigned char pwm4, unsigned char pwm5,
                 unsigned char pwm6, unsigned char pwm7, unsigned char pwm8){
    if (pwm0 > pwm_counter || pwm0 == 255)
    {PWM_OUT_0 = HIGH;}
    else
    {PWM_OUT_0 = LOW;}

    if (pwm1 > pwm_counter || pwm1 == 255)
    {PWM_OUT_1 = HIGH;}
    else
    {PWM_OUT_1 = LOW;}

    if (pwm2 > pwm_counter || pwm2 == 255)
    {PWM_OUT_2 = HIGH;}
    else
    {PWM_OUT_2 = LOW;}

    if (pwm3 > pwm_counter || pwm3 == 255)
    {PWM_OUT_3 = HIGH;}
    else
    {PWM_OUT_3 = LOW;}

    if (pwm4 > pwm_counter || pwm4 == 255)
    {PWM_OUT_4 = HIGH;}
    else
    {PWM_OUT_4 = LOW;}

    if (pwm5 > pwm_counter || pwm5 == 255)
    {PWM_OUT_5 = HIGH;}
    else
    {PWM_OUT_5 = LOW;}

    if (pwm6 > pwm_counter || pwm6 == 255)
    {PWM_OUT_6 = HIGH;}
    else
    {PWM_OUT_6 = LOW;}

    if (pwm7 > pwm_counter || pwm7 == 255)
    {PWM_OUT_7 = HIGH;}
    else
    {PWM_OUT_7 = LOW;}

    if (pwm8 > pwm_counter || pwm8 == 255)
    {PWM_OUT_8 = HIGH;}
    else
    {PWM_OUT_8 = LOW;}

    pwm_counter++;
}

void PcCommands(){
//Azzero il contatore caratteri, per poter ricevere nuovi comandi
    rx_counter = 0;
//Controllo il primo byte ricevuto, che contiene il comando
//Gli altri 3 byte contengono i parametri
    switch (OP_BYTE){
        case LEFT_BAR:
            pcLeftRed = RED_BYTE;
            pcLeftGreen = GREEN_BYTE;
            pcLeftBlue = BLUE_BYTE;
            break;
        case RIGHT_BAR:
            pcRightRed = RED_BYTE;
            pcRightGreen = GREEN_BYTE;
            pcRightBlue = BLUE_BYTE;
            break;
        case AUX_BAR:
            pcAuxRed = RED_BYTE;
            pcAuxGreen = GREEN_BYTE;
            pcAuxBlue = BLUE_BYTE;
            break;
        case COMMAND:
            if (CMD_BYTE <= 2 && power == ON){
                SendACK;
                pcMode = CMD_BYTE;
            }
            else if(CMD_BYTE == PWR_OFF){
                SendACK;
                power = OFF;
                TogglePower();
            }
            else if(CMD_BYTE == PWR_ON){
                SendACK;
                power = ON;
                TogglePower();
            }
            else{SendNACK;}
            break;
        case READ_AUTHOR:
            SendAuthor;
            break;
        case READ_VERSION:
            SendVersion;
            break;
        case READ_HWVERSION:
            SendHwVersion;
            break;
        case READ_SYSTEM:
            SendSystem;
            break;
        case READ_LAST_UPDATE:
            SendLastUpdate;
            break;
        default:
            SendNACK;
            break;
    }
}

void AdConversion(){
//Primo step della conversione: leggo il potenziometro, confermo la lettura
//e passo al canale dei pulsanti
    if (AdcOperation == 0){
        potValue = ADC_RESULT;
        potIsValid = TRUE;
        SELECT_ADC_CHANNEL(BTN_INPUT);
    }
//Secondo step: avvio conversione
    if (AdcOperation == 1){
        TMR0IE = 0;
        GO = 1;
        TMR0IE = 1;
        potIsValid = FALSE;
    }
//Terzo step: leggo i pulsanti, confermo la lettura e passo al canale del pot.
    if (AdcOperation == 2){
        btnValue = ADC_RESULT;
        btnIsValid = TRUE;
        SELECT_ADC_CHANNEL(POT_INPUT);
    }
//Quarto step: avvio conversione
    if (AdcOperation == 3){
        TMR0IE = 0;
        GO = 1;
        TMR0IE = 1;
        btnIsValid = FALSE;
    }
//Incremento il contatore dello step di conversione
    AdcOperation++;
    if (AdcOperation == 4){AdcOperation = 0;}
}

void AnalogBTN(unsigned int btnValue){
//Controllo se la lettura rientra nei valori previsti dai pulsanti, e se il
//comando non  gi stato eseguito
    if(btnValue >= BTN1_MIN && btnValue <= BTN1_MAX && !cmdOk){
//Se il pulsante  appena stato premuto lo imposto come selezionato e azzero
//il conteggio antirimbalzo
        if (btnSelected != BTN1){
            btnSelected = BTN1;
            btnDebounceCounter = 0;
        }
//Viceversa incremento il conteggio
        else{
            btnDebounceCounter ++;
        }
    }

    if(btnValue >= BTN2_MIN && btnValue <= BTN2_MAX && !cmdOk){
        if (btnSelected != BTN2){
            btnSelected = BTN2;
            btnDebounceCounter = 0;
        }
        else{
            btnDebounceCounter ++;
        }
    }

    if(btnValue >= BTN3_MIN && btnValue <= BTN3_MAX && !cmdOk){
        if (btnSelected != BTN3){
            btnSelected = BTN3;
            btnDebounceCounter = 0;
        }
        else{
            btnDebounceCounter ++;
        }
    }
//Quando il conteggio arriva al valore stabilito, passo ad eseguire il comando
    if (btnDebounceCounter == DEBOUNCE_COUNT){
//Azzero il contatore e setto il flag del comando eseguito
        btnDebounceCounter = 0;
        cmdOk = 1;
//Controllo quale pulsante  stato premuto ed eseguo l'operazione associata
        switch (btnSelected){
            case EFFECT:
                effect =! effect;
                break;
            case POWER:
                power =! power;
                TogglePower();
                break;
            case MODE:
                mode =! mode;
                break;
        }
    }
//Quando il pulsante viene rilasciato azzero il flag per ricevere un nuovo
//comando
    if (btnValue >= DEFAULT_VALUE){
        cmdOk = 0;
    }
}

void RainbowRGB(unsigned int analog){
    if (analog < 2)
    {
        effectRed = 255;
        effectGreen = 0;
        effectBlue = 0;
    }
    if (analog >= 2 && analog <= 171)
    {
        effectRed = 255;
        effectGreen = 0;
        effectBlue = ((analog - 1) * 15)/10;
    }
    if (analog >= 172 && analog <= 341)
    {
        effectRed = 255-(((analog - 171) * 15)/10);
        effectGreen = 0;
        effectBlue = 255;
    }
    if (analog >= 342 && analog <= 511)
    {
        effectRed = 0;
        effectGreen = ((analog - 341) * 15)/10;
        effectBlue = 255;
    }
    if (analog >= 512 && analog <= 681)
    {
        effectRed = 0;
        effectGreen = 255;
        effectBlue = 255-(((analog - 511) * 15)/10);
    }
    if (analog >= 682 && analog <= 851)
    {
        effectRed = ((analog - 681) * 15)/10;
        effectGreen = 255;
        effectBlue = 0;
    }
    if (analog >= 852 && analog <= 1021)
    {
        effectRed = 255;
        effectGreen = 255;
        effectBlue = ((analog - 851) * 15)/10;
    }
    if (analog >= 1022)
    {
        effectRed = 255;
        effectGreen = 255;
        effectBlue = 255;
    }
}

void SoftRGB(unsigned int analog){
    if(analog == 0){
        effectRed = 0;
        effectGreen = 0;
        effectBlue = 0;
    }
    if(analog >= 1 && analog <= 73){
        effectRed = (analog * 3 );
        effectGreen = 0;
        effectBlue = 0;
    }
    if(analog >= 74 && analog <= 146){
        effectRed = 255 - (((analog - 73) *  3 ));
        effectGreen = 0;
        effectBlue = 0;
    }
    if(analog >= 147 && analog <= 219){
        effectRed = ((analog - 146) *  3 );
        effectGreen = 0;
        effectBlue = effectRed;
    }
    if(analog >= 220 && analog <= 292){
        effectRed = 255 - (((analog - 219) *  3 ));
        effectGreen = 0;
        effectBlue = effectRed;
    }
    if(analog >= 293 && analog <= 365){
        effectRed = 0;
        effectGreen = 0;
        effectBlue = ((analog - 292) *  3 );
    }
    if(analog >= 366 && analog <= 438){
        effectRed = 0;
        effectGreen = 0;
        effectBlue = 255 - (((analog - 365) *  3 ));
    }
    if(analog >= 439 && analog <= 511){
        effectRed = 0;
        effectGreen = ((analog - 438) *  3 );
        effectBlue = effectGreen;
    }
    if(analog >= 512 && analog <= 584){
        effectRed = 0;
        effectGreen = 255 - (((analog - 511) *  3 ));
        effectBlue = effectGreen;
    }
    if(analog >= 585 && analog <= 657){
        effectRed = 0;
        effectGreen = ((analog - 584) *  3 );
        effectBlue = 0;
    }
    if(analog >= 658 && analog <= 730){
        effectRed = 0;
        effectGreen = 255 - (((analog - 657) *  3 ));
        effectBlue = 0;
    }
    if(analog >= 731 && analog <= 803){
        effectRed = ((analog - 730) *  3 );
        effectGreen = effectRed;
        effectBlue = 0;
    }
    if(analog >= 804 && analog <= 876){
        effectRed = 255 - (((analog - 803) *  3 ));
        effectGreen = effectRed;
        effectBlue = 0;
    }
    if(analog >= 877 && analog <= 949){
        effectRed = ((analog - 876) *  3 );
        effectGreen = effectRed;
        effectBlue = effectRed;
    }
    if(analog >= 950 && analog <= 1022){
        effectRed = 255 - (((analog - 949) *  3 ));
        effectGreen = effectRed;
        effectBlue = effectRed;
    }
}

void TogglePower(){
//Spegnimento controller
    if (power == OFF){
//Salvataggio configurazione corrente
        GIE = FALSE;
        cfgVar = mode;
        eeprom_write(MODE_ADDRESS, cfgVar);
        cfgVar = effect;
        eeprom_write(EFFECT_ADDRESS, cfgVar);
        GIE = TRUE;
//Spengo i led
        pcMode = PWR_OFF;
//Invio notifica su seriale
        SendPwrOff;
    }
//Accensione controller
    else{
//Carico cfg salvata
        GIE = FALSE;
        cfgVar = eeprom_read(MODE_ADDRESS);
        mode = cfgVar;
        cfgVar = eeprom_read(EFFECT_ADDRESS);
        effect = cfgVar;
        GIE = TRUE;
//Abilito led in default (controllo locale)
        pcMode = PC_MODE_OFF;
//Invio notifica su seriale
        SendPwrOn;
    }
}