/*******************************************************************************
 * RGB Led Controller - header file
 * Autore: Matteo Garia
 * Data Creazione: 19/04/2012
 * Versione: 1.1 (07/10/2012)
 * ****************************************************************************/

/*******************************************************************************
 * Informazioni progetto
 * ****************************************************************************/
#define AUTHOR "Matteo Garia\n\r"
#define SYSTEM "RGB_Led_Controller\n\r"
#define VERSION "Firmware Version 1.1\n\r"
#define HWVERSION "Hardware ver. 1.0 rev. 0\n\r"
#define LAST_UPDATE "Updated on: 07/10/2012\n\r"

/*******************************************************************************
 * Configurazione di sistema
 * ****************************************************************************/
#include <htc.h>
__CONFIG(WDTE_OFF & LVP_OFF & FOSC_HS & MCLRE_OFF & DEBUG_OFF);

//preload timer 0
#define TMR0_PRELOAD 26

//setup porta seriale
#define BAUD 38400
#define FOSC 20000000L
#define NINE 0
#include <usart.c>

//configurazione Program Counter interno
#define PC_PRELOAD 200
unsigned char programCounter = PC_PRELOAD;
#define RESET_PROGRAM_COUNTER programCounter = PC_PRELOAD;

//Flag per la registrazione degli interrupt
static bit main_int;
static bit usart_int;

/*******************************************************************************
 * Funzione Macro configurazione Special Function Registers
 * ****************************************************************************/
#define SetupSFR()\
    TRISA=0b00000011;\
    PORTA=0x00;\
    TRISB=0xb00100100;\
    PORTB=0x00;\
    OPTION_REG=0b01001000;\
    INTCON=0b11100000;\
    RCIE = 1

/*******************************************************************************
 * Configurazione ADC
 * ****************************************************************************/
#define SetupADC()\
ANSEL = 0b00000011;\
ADCON0 = 0b10000001;\
ADCON1 = 0b10000000

//Macro per la selezione dell'ingresso analogico
#define SELECT_ADC_CHANNEL(a) ADCON0 = (ADCON0 & 0b11000111) | a<<3
//Risultato della conversione
#define ADC_RESULT ADRESL + (ADRESH<<8)

//Ingressi utilizzati
#define POT_INPUT 1
#define BTN_INPUT 0

//Funzione di acquisizione
void AdConversion();
unsigned char AdcOperation;
//Risultati della conversione
unsigned int potValue;
unsigned int btnValue;
static bit potIsValid;
static bit btnIsValid;
/*******************************************************************************
 * Varie
 * ****************************************************************************/
#define HIGH 1
#define LOW 0
#define TRUE 1
#define FALSE 0

/*******************************************************************************
 * Collegamenti barre a led
 * ****************************************************************************/
#define LEFT_RED RA4
#define LEFT_GREEN RA2
#define LEFT_BLUE RA3

#define RIGHT_RED RB0
#define RIGHT_GREEN RB3
#define RIGHT_BLUE RB1

#define AUX_RED RB4
#define AUX_GREEN RB7
#define AUX_BLUE RB6

/*******************************************************************************
 * Controllo da seriale
 * ****************************************************************************/
//Durata timeout
#define TIMEOUT 200

//Elenco comandi in lettura
#define READ_AUTHOR 'a'         //0x61 0x00 0x00 0x00
#define READ_VERSION 'v'        //0x76 0x00 0x00 0x00
#define READ_SYSTEM 's'         //0x73 0x00 0x00 0x00
#define READ_HWVERSION 'h'      //0x68 0x00 0x00 0x00
#define READ_LAST_UPDATE 'd'    //0x64 0x00 0x00 0x00
//Elenco comandi in scrittura
#define LEFT_BAR 'l'        //0x6c RED GREEN BLUE
#define RIGHT_BAR 'r'       //0x72 RED GREEN BLUE
#define AUX_BAR 'x'         //0x78 RED GREEN BLUE
#define COMMAND 'c'         //0x63 0x00 0x00 CMD

//Operazioni ammesse con comando 'c'
#define PC_MODE_OFF 0           //disabilita il controllo da pc
#define PC_MODE_ON 1            //abilita il controllo da pc su barre L ed R
#define AUX_BAR_PC_MODE_ON 2    //abilita il controllo da pc su tutte le barre
#define PWR_OFF 3               //spegne il controller
#define PWR_ON 4                //accende il controller

unsigned char pcMode;

//Nomi mnemonici dei byte ricevuti
#define OP_BYTE rx_chars[0]
#define RED_BYTE rx_chars[1]
#define GREEN_BYTE rx_chars[2]
#define BLUE_BYTE rx_chars[3]
#define CMD_BYTE rx_chars[3]

//Messaggi di rispsota ai comandi
#define SendAuthor printf(AUTHOR)
#define SendVersion printf(VERSION)
#define SendSystem printf(SYSTEM)
#define SendHwVersion printf(HWVERSION)
#define SendLastUpdate printf(LAST_UPDATE)
//Messaggio di errore
#define ERROR printf("timeout\n\r")
//Conferma validit comando
#define SendACK printf ("ACK\n\r")
#define SendNACK printf ("NACK\n\r")
//Messaggi stato alimentazione
#define SendPwrOff printf ("power OFF\n\r")
#define SendPwrOn printf ("power ON\n\r")

//Array dove vengono memorizzati i caratteri ricevuti
unsigned char rx_chars[5];
//Contatore caratteri
unsigned char rx_counter;
//Conteggio per il timeout
unsigned char com_timeout;

/*******************************************************************************
 * Funzione che gestisce la ricezione dei comandi da seriale
 * Parametri:
 *
 * Ritorno:
 *      void
 * ****************************************************************************/
void PcCommands();

/*******************************************************************************
 * Controllo da pulsanti
 * ****************************************************************************/
//Valori ADC teorici relativi ad ogni pulsante
#define BTN1_VALUE 327
#define BTN2_VALUE 512
#define BTN3_VALUE 704
#define DEFAULT_VALUE 1000
//Tolleranza ammessa dei pulsanti (+/- rispetto teorico)
#define BTN_RANGE 30
//Numero conteggi antirimbalzo
#define DEBOUNCE_COUNT 2
//definizione pulsanti
#define BTN1 1
#define BTN2 2
#define BTN3 3
//Nomi mnemonici
#define EFFECT BTN1
#define POWER BTN2
#define MODE BTN3

//Calcolo valori MIN e MAX ammessi per ogni pulsante (non toccare)
#define BTN1_MIN (BTN1_VALUE - BTN_RANGE)
#define BTN1_MAX (BTN1_VALUE + BTN_RANGE)
#define BTN2_MIN (BTN2_VALUE - BTN_RANGE)
#define BTN2_MAX (BTN2_VALUE + BTN_RANGE)
#define BTN3_MIN (BTN3_VALUE - BTN_RANGE)
#define BTN3_MAX (BTN3_VALUE + BTN_RANGE)

/*******************************************************************************
 * Funzione che gestisce pulsanti con ingresso analogico
 * Parametri:
 *      btnValue: lettura del canale ADC su cui sono collegati i pulsanti
 * Ritorno:
 *      void
 * ****************************************************************************/
//Variabile conteggio antirimbalzo
unsigned char btnDebounceCounter;
//Variabile indicante il pulsante premuto
unsigned char btnSelected;
//Flag per comando eseguito
static bit cmdOk;

void AnalogBTN(unsigned int btnValue);

/*******************************************************************************
 * Funzione che genera 9 segnali PWM via software
 * Parametri:
 *      pwm0..8 valore pwm della rispettiva uscita [0..255]
 * Ritorno:
 *      void
 * ****************************************************************************/

//variabile di conteggio per la generazione PWM. Non modificare
unsigned char pwm_counter;

#define PWM_OUT_0 LEFT_RED
#define PWM_OUT_1 LEFT_GREEN
#define PWM_OUT_2 LEFT_BLUE
#define PWM_OUT_3 RIGHT_RED
#define PWM_OUT_4 RIGHT_GREEN
#define PWM_OUT_5 RIGHT_BLUE
#define PWM_OUT_6 AUX_RED
#define PWM_OUT_7 AUX_GREEN
#define PWM_OUT_8 AUX_BLUE

void SoftwarePwm(unsigned char pwm0, unsigned char pwm1, unsigned char pwm2,
                 unsigned char pwm3, unsigned char pwm4, unsigned char pwm5,
                 unsigned char pwm6, unsigned char pwm7, unsigned char pwm8);

//Variabili valori PWM ricevuti da pc
unsigned char pcLeftRed;
unsigned char pcLeftGreen;
unsigned char pcLeftBlue;

unsigned char pcRightRed;
unsigned char pcRightGreen;
unsigned char pcRightBlue;

unsigned char pcAuxRed;
unsigned char pcAuxGreen;
unsigned char pcAuxBlue;

//Variabili valori PWM degli effetti
unsigned char effectRed;
unsigned char effectGreen;
unsigned char effectBlue;

/*******************************************************************************
 * Funzioni per gli effetti di luce predefiniti (Rainbow, Soft)
 *
 * Parametri:
 *      analog: controllo a 10 bit della funzione
 * Ritorno:
 *      void
 * ****************************************************************************/
void RainbowRGB(unsigned int analog);
void SoftRGB(unsigned int analog);

//Variabile dove viene scritto il valore istantaneo dell'effetto
unsigned int effectValue;
//contatore per la temporizzazione degli effetti
unsigned int autoTimer;

/*******************************************************************************
 * Variabili di stato del sistema (selezione effetti/modalit)
 * ****************************************************************************/

static bit effect;
#define RAINBOW 0
#define SOFT 1

static bit mode;
#define MANUAL 0
#define AUTO 1

static bit power;
#define OFF 0
#define ON 1

/*******************************************************************************
 * Salvataggio impostazioni e gestione accensione/spegnimento
 * ****************************************************************************/
unsigned char cfgVar;
#define MODE_ADDRESS 0
#define EFFECT_ADDRESS 1

void TogglePower();
